#import "SpeechSynthesisPatch.h"
#import <AppKit/NSSpeechSynthesizer.h>

@implementation SpeechSynthesisPatch : QCPatch

+ (int)executionMode
{
	return 1;
}
+ (BOOL)allowsSubpatches
{
	return NO;
}
+ (int)timeMode
{
	return 1;
}


- (id)initWithIdentifier:(id)fp8
{
	id z=[super initWithIdentifier:fp8];
	if(z)
	{
		speech = nil;

		// currently, we have continuous, per-trigger, and per-trigger(hold) modes
		[inputCountMode setMaxIndexValue:2];
	}
	return z;
}


- (id)setup:(id)fp8
{
	speechLock = [[NSLock alloc] init];
	oldValue = FALSE;
	return fp8;
}
- (void)cleanup:(id)fp8
{
}


- (void)enable:(id)fp8
{
}
- (void)disable:(id)fp8
{
	if(speech != nil)
		[speech stopSpeaking];
}

- (BOOL)execute:(id)fp8 time:(double)fp12 arguments:(id)fp20
{
	//NSLog(@"execute: %i %i %i\n",[inputTrigger booleanValue], oldValue, speech);
	[speechLock lock];
	if([inputTrigger booleanValue] == TRUE && oldValue == FALSE && speech == nil)
	{
		// speak the text.
		//NSLog(@"trying to talk\n");
		// if we're in per-trigger (hold) mode, we need to clean on trigger.  This is safe to do if we're in hold mode as well, so we're
		// actually only going to see if we're in continuous mode
		if([inputCountMode indexValue] != 0)
		{
			spokenWords = 0;
			[outputWords setDoubleValue:0];
		}
		speech = [[NSSpeechSynthesizer alloc] init];
		[speech setVoice:[[NSSpeechSynthesizer availableVoices] objectAtIndex:[inputVoice indexValue]]];
		[speech setUsesFeedbackWindow:FALSE];
		[speech setDelegate:self];
		[speech startSpeakingString:[inputString stringValue]];
	}
	if([speech isSpeaking] == NO)
	{
		[speech release];
		speech = nil;
	}
	[speechLock unlock];
	
	oldValue = [inputTrigger booleanValue];
		
	return YES;
}

- (void)speechSynthesizer:(NSSpeechSynthesizer *)sender didFinishSpeaking:(BOOL)success
{
	//NSLog(@"didFinishSpeaking\n");
	// clean up
	// if we're in per-trigger (non-hold) mode, we need to clean when we're done talking.
	if([inputCountMode indexValue] == 1)
	{
		spokenWords = 0;
		[outputWords setDoubleValue:0];
	}
	[speechLock lock];
		[speech release];
		speech = nil;
	[speechLock unlock];
}

- (void)speechSynthesizer:(NSSpeechSynthesizer *)sender willSpeakWord:(NSRange)wordToSpeak ofString:(NSString *)text
{
	//NSLog(@"willSpeakWord\n");
	++spokenWords;
	[outputWords setDoubleValue:(double)spokenWords];
}

- (void)speechSynthesizer:(NSSpeechSynthesizer *)sender willSpeakPhoneme:(short)phonemeOpcode
{
	[outputPhoneme setDoubleValue:(double)phonemeOpcode];
	//NSLog(@"willSpeakPhoeneme: %i\n",phonemeOpcode);
}

@end
